@@ -1,5 +1,5 @@
 # From SamplerCompare, (c) 2010 Madeleine Thompson
-# $Id: compare-samplers.R 2756 2011-03-16 11:36:08Z mthompson $
+# $Id: compare-samplers.R 3071 2011-05-01 18:06:48Z mthompson $
 
 # compare-samplers.R contains compare.samplers and its support
 # function eval.sampler, which runs an individual simulation.
@@ -115,62 +115,24 @@ eval.sampler <- function(dist, sampler, sample.size, tuning.param,
     set.seed(seed)
   }
 
-  # Initialize value to return.
+  # Run the sampler started at a random point on the unit hypercube
+  # or an initial point generated by the distribution if available.
 
-  rv <- list(dist=dist$name, ndim=dist$ndim,
-             sampler=attr(sampler, 'name'), tuning=tuning.param,
-             act=NA, act.025=NA, act.975=NA,
-             act.y=NA, act.y.025=NA, act.y.975=NA,
-             evals=NA, grads=NA, cpu=NA, err=NA, aborted=TRUE)
-
-  # Figure out plotmath names for sampler and distribution,
-  # defaulting to the regular names.
-
-  a <- attr(sampler, 'name.expression')
-  rv$sampler.expr <-
-    ifelse(is.null(a), sprintf('plain("%s")', rv$sampler), a)
-  a <- dist$name.expression
-  rv$dist.expr <-
-    ifelse(is.null(a), sprintf('plain("%s")', rv$dist), a)
-
-  # Run the sampler started at a random point on the unit hypercube.
-
-  x0 <- runif(dist$ndim)
+  if (is.null(dist$initial))
+    x0 <- runif(dist$ndim)
+  else
+    x0 <- dist$initial()
   timings <- system.time(
     S <- sampler(target.dist=dist, x0=x0, sample.size=sample.size,
                  tuning=tuning.param) )
   stopifnot(!is.null(S$X) && !is.null(S$evals))
 
-  # Discard the first 20% of the sample and compute the autocorrelation
-  # time on what remains.
+  # Compute value to return.
 
-  discard.obs <- floor(sample.size * burn.in)
-  if (nrow(S$X)>discard.obs) {  # enough steps for act
-    chain <- S$X[discard.obs:nrow(S$X),,drop=FALSE]
-    acts <- ar.act(chain, true.mean=dist$mean)
-
-    rv$act <- acts$act
-    rv$act.025 <- acts$act.025
-    rv$act.975 <- acts$act.975
-
-    y <- apply(chain, 1, dist$log.density)
-    acts.y <- ar.act(y, true.mean=dist$mean.log.dens)
-    rv$act.y <- acts.y$act
-    rv$act.y.025 <- acts.y$act.025
-    rv$act.y.975 <- acts.y$act.975
-
-    # Fill in remaining fields in return value with simulation results.
-
-    rv$evals <- S$evals / nrow(S$X)
-    rv$grads <- ifelse(is.null(S$grads), NA, S$grads/nrow(S$X))
-    rv$cpu <- timings[['elapsed']] / nrow(S$X)
-    rv$aborted <- nrow(S$X) < sample.size
-
-    # Compute the error in the mean if possible.
-
-    if (!is.null(dist$mean))
-      rv$err <- sqrt(sum((dist$mean - colMeans(chain))^2))
-  }
+  rv <- simulation.result(dist, attr(sampler, 'name'), S$X,
+    S$evals, S$grads, tuning.param, timings[['elapsed']], burn.in,
+    sampler.expr=attr(sampler, 'name.expression'),
+    aborted = nrow(S$X) < sample.size)
 
   # Print results of simulation on terminal if requested.
 
@@ -192,7 +154,7 @@ eval.sampler <- function(dist, sampler, sample.size, tuning.param,
       lock(completed.lock)
 
     load(completed.file)
-    sc.new <- rbind(sampler.comparison, data.frame(rv))
+    sc.new <- rbind(sampler.comparison, rv)
     row.names(sc.new) <- 1:nrow(sc.new)
     attr(sc.new, 'start.time') <- attr(sampler.comparison, 'start.time')
     attr(sc.new, 'start.stamp') <- attr(sampler.comparison, 'start.stamp')
@@ -208,3 +170,68 @@ eval.sampler <- function(dist, sampler, sample.size, tuning.param,
 
   return(rv)
 }
+   
+simulation.result <- function(target.dist, sampler.name, X,
+    evals=NULL, grads=NULL, tuning=NULL, cpu=NULL, burn.in=0.2, y=NULL,
+    sampler.expr=sprintf("plain('%s')", sampler.name),
+    aborted=NA) {
+
+  # Cast X to a matrix.  Necessary when it is an mcmc object from coda.
+
+  X <- as.matrix(X)
+  
+  # Check and chop off burn-in segments of chains.
+
+  stopifnot(target.dist$ndim==ncol(X))
+  first.obs <- ceiling(burn.in*nrow(X))
+  chain <- X[first.obs:nrow(X),,drop=FALSE]
+
+  # Initialize return value.
+
+  nNA <- as.numeric(NA)
+  rv <- list(dist=target.dist$name, dist.expr=target.dist$name.expression,
+             ndim=target.dist$ndim, sampler=sampler.name,
+             sampler.expr=sampler.expr,
+             tuning=tuning, act=nNA, act.025=nNA, act.975=nNA,
+             act.y=nNA, act.y.025=nNA, act.y.975=nNA,
+             evals=ifelse(is.null(evals), nNA, evals/nrow(X)),
+             grads=ifelse(is.null(grads), nNA, grads/nrow(X)),
+             cpu=ifelse(is.null(cpu), nNA, cpu/nrow(X)),
+             err=nNA, aborted=aborted)
+
+  if (is.null(rv$dist.expr))
+    rv$dist.expr <- sprintf('plain("%s")', rv$dist)
+
+  if (is.null(sampler.expr))
+    rv$sampler.expr <- sprintf("plain('%s')", sampler.name)
+
+  # Fill in maximal autocorrelation times for components of chain.
+
+  acts <- ar.act(chain, true.mean=target.dist$mean)
+  rv$act <- acts$act
+  rv$act.025 <- acts$act.025
+  rv$act.975 <- acts$act.975
+
+  # Fill in autocorrelation times for log density if the caller
+  # specifies the log density function or passes in log density states
+  # explicitly.
+
+  if (is.null(y) && !is.null(target.dist$log.density))
+    y <- apply(chain, 1, target.dist$log.density)
+  if (!is.null(y)) {
+    acts.y <- ar.act(y, true.mean=target.dist$mean.log.dens)
+    rv$act.y <- acts.y$act
+    rv$act.y.025 <- acts.y$act.025
+    rv$act.y.975 <- acts.y$act.975
+  }
+
+  # Compute error in sample mean if the true mean is known.
+
+  if (!is.null(target.dist$mean))
+    rv$err <- sqrt(sum((target.dist$mean - colMeans(chain))^2))
+
+  # Return data as a data frame with strings for factors so rbind
+  # works as expected.
+  
+  return(as.data.frame(rv, stringsAsFactors=FALSE))
+}
